<?php

// autoload_static.php @generated by Composer

namespace Composer\Autoload;

class ComposerStaticInita55c1613fd11ee6fb2b03d8dfab5c54d
{
    public static $files = array (
        '3109cb1a231dcd04bee1f9f620d46975' => __DIR__ . '/..' . '/paragonie/sodium_compat/autoload.php',
    );

    public static $prefixLengthsPsr4 = array (
        'T' => 
        array (
            'Telnyx\\' => 7,
        ),
    );

    public static $prefixDirsPsr4 = array (
        'Telnyx\\' => 
        array (
            0 => __DIR__ . '/../..' . '/lib',
            1 => __DIR__ . '/..' . '/telnyx/telnyx-php/lib',
        ),
    );

    public static $classMap = array (
        'Composer\\InstalledVersions' => __DIR__ . '/..' . '/composer/InstalledVersions.php',
    );

    public static function getInitializer(ClassLoader $loader)
    {
        return \Closure::bind(function () use ($loader) {
            $loader->prefixLengthsPsr4 = ComposerStaticInita55c1613fd11ee6fb2b03d8dfab5c54d::$prefixLengthsPsr4;
            $loader->prefixDirsPsr4 = ComposerStaticInita55c1613fd11ee6fb2b03d8dfab5c54d::$prefixDirsPsr4;
            $loader->classMap = ComposerStaticInita55c1613fd11ee6fb2b03d8dfab5c54d::$classMap;

        }, null, ClassLoader::class);
    }
}
