<?php

namespace Telnyx;

/**
 * Class OtaUpdate
 *
 * @package Telnyx
 */
class OtaUpdate extends ApiResource
{
    const OBJECT_NAME = "ota_update";

    use ApiOperations\All;
    use ApiOperations\Retrieve;
    
}
