<?php
/**
 * @copyright 2021 Double Bastion LLC <www.doublebastion.com>
 *
 * @author Double Bastion LLC
 *
 * @license GNU AGPL version 3 or any later version
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU AFFERO GENERAL PUBLIC LICENSE for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

declare(strict_types=1);

namespace OCA\SMSRelentless\Settings;

use OCP\IConfig;
use OCP\AppFramework\Http\TemplateResponse;
use OCP\Settings\ISettings;


class Personal implements ISettings {

        /** @var IConfig */
	private IConfig $config;

	public function __construct(IConfig $config) {
		$this->config = $config;
	}


        public function getForm(): TemplateResponse {
		return new TemplateResponse('sms_relentless', 'settings');
        }

        /**
         * @return string the section ID, e.g. 'sharing'
         */
        public function getSection(): string {
                return 'sms_relentless';
        }

        /**
         * @return int whether the form should be rather on the top or bottom of
         * the admin section. The forms are arranged in ascending order of the
         * priority values. It is required to return a value between 0 and 100.
         *
         * keep the server setting at the top, right after "server settings"
         */
        public function getPriority(): int {
                return 11;
        }
}

